import React from 'react';
import { WithAsProps, RsRefForwardingComponent } from '../@types/common';
import Tab from './Tab';
export interface TabsProps extends WithAsProps {
    /**
     * The tabs appearance style.
     */
    appearance?: 'tabs' | 'subtle';
    /**
     * Mark the Tab with a matching `eventKey` as active.
     */
    activeKey?: string;
    /**
     * The default active tabKey.
     */
    defaultActiveKey?: string;
    /**
     * Reversed display.
     * @default false
     */
    reversed?: boolean;
    /**
     * Vertical display.
     * @default false
     */
    vertical?: boolean;
    /**
     * The HTML id attribute, which should be unique.
     * @default use generated id
     */
    id?: string;
    /**
     * Callback fired when a Tab is selected.
     * @param eventKey
     * @param event
     * @returns
     */
    onSelect?: (eventKey: string | undefined, event: React.SyntheticEvent) => void;
}
interface TabsComponent extends RsRefForwardingComponent<'div', TabsProps> {
    Tab: typeof Tab;
}
/**
 * Tabs are a set of layered sections of content, known as tab panels, that display one panel of content at a time.
 *
 * @version 5.53.0
 * @see https://rsuitejs.com/components/tabs
 */
declare const Tabs: TabsComponent;
export default Tabs;
