import React from 'react';
import { TypeAttributes } from '../@types/common';
import type { Schema, CheckResult } from 'schema-typed';
import type { FieldRuleType } from './hooks/useSchemaModel';
declare type RecordAny = Record<string, any>;
interface TrulyFormContextValue<T = RecordAny, errorMsgType = any, E = {
    [P in keyof T]?: errorMsgType;
}> {
    formError: E;
    nestedField: boolean;
    getCombinedModel: () => Schema;
    removeFieldValue: (name: string) => void;
    removeFieldError: (name: string) => void;
    removeFieldRule: (name: string) => void;
    pushFieldRule: (name: string, fieldRule: FieldRuleType) => void;
    onFieldError: (name: string, fieldError: string | CheckResult) => void;
    onFieldChange: (name: string, value: any, event: React.SyntheticEvent) => void;
    onFieldSuccess: (name: string) => void;
}
declare type ExternalPropsContextValue = {
    checkTrigger?: TypeAttributes.CheckTrigger;
    errorFromContext?: boolean;
    readOnly?: boolean;
    plaintext?: boolean;
    disabled?: boolean;
};
declare type InitialContextType = Partial<Record<keyof TrulyFormContextValue, undefined>>;
export declare type FormContextValue<T = RecordAny, errorMsgType = any> = (TrulyFormContextValue<T, errorMsgType> | InitialContextType) & ExternalPropsContextValue;
export declare const FormContext: React.Context<FormContextValue<RecordAny, any>>;
export declare const FormValueContext: React.Context<RecordAny | undefined>;
export declare const FormProvider: React.Provider<FormContextValue<RecordAny, any>>;
export declare const FormValueProvider: React.Provider<RecordAny | undefined>;
export declare function useFormContext<T = RecordAny, E = any>(): FormContextValue<T, E>;
export default FormContext;
